#include    "main.h"
#include    "goto.h"

void ED_Init (struct RegisterAxis *ra, struct RegisterAxis *dec){
	
	//ra->K1  = (9024000/100) / (float)RA_TOTAL_STEPS; //     c() EQMOD
	//dec->K1 = (9024000/100) / (float)DEC_TOTAL_STEPS;
	ra->K1  = (float)RA_TOTAL_STEPS / 9024000; //     c() EQMOD
	dec->K1 = (float)DEC_TOTAL_STEPS / 9024000;
	
	ra->K2  = (2.31*9024000) / (float)RA_TOTAL_STEPS;
	dec->K2 = (2.31*9024000) / (float)DEC_TOTAL_STEPS;
	
	ra->BeginBraking  = (u32)(25000 * ra->K1); //       GoTo   800
	dec->BeginBraking = (u32)(25000 * dec->K1);
	
	ra->GotoBegin800x  = (u32)(0xFFFF * ra->K1); // GoTo     800
	dec->GotoBegin800x = (u32)(0xFFFF * dec->K1);
	
	ra -> Speed_1x  = (u32)(620 * ra->K2);
	dec -> Speed_1x = (u32)(620 * dec->K2);
	
	ra -> Speed  = ra -> Speed_1x;
	dec -> Speed = dec -> Speed_1x;
	
	ra->Mode  = 1;
	dec->Mode = 1;
	
	//      Step/Dir
#if defined (EQDRIVE) || (defined EQDRIVE_PRE)
	EN_RA_UP;
	DIR_RA_UP;
	EN_DEC_UP;
	DIR_DEC_UP;
#endif
	
#ifdef EQDRIVE_PRE
	PW_RA_DW;
	PW_DEC_DW;
	DAC->DHR12R1 = (u16)(VREF_RA * 1.240);
	TIM2->CCR3 = (u16)(VREF_DEC / 10);
#endif
	
#ifdef EQDRIVE
	//    2.82V N=2.82V*4095/2.94V
	DAC->DHR12R1 = (u16)(VREF_RA * 1.393);
	DAC->DHR12R2 = (u16)(VREF_DEC * 1.393);
#endif
#ifdef EQDRIVE_PRO
	DRV8813_Init();
#endif
}

void ED_InitSpeedRA (struct RegisterAxis *axis){
	if(axis->Enable){
		EN_RA_DW;
#ifdef EQDRIVE_PRO		
		DACAY_RA_UP;
#endif
	}
	switch (axis->Mode){
#if defined (EQDRIVE) || (defined EQDRIVE_PRE)
	case 0: case 1: {DIR_RA_UP; TIM4->PSC = 160 - 1; TIM4->ARR = axis->Speed; break;}
	case 2: case 3: {DIR_RA_DW; TIM4->PSC = 160 - 1; TIM4->ARR = axis->Speed; break;}
	case 4: case 5: {DIR_RA_UP; TIM4->ARR = axis->Speed; break;}
	case 6: case 7: {DIR_RA_DW; TIM4->ARR = axis->Speed; break;}
#endif
#ifdef EQDRIVE_PRO
	case 0: case 1: {DRV8813_Dir_RA(1); TIM4->PSC = 160 - 1; TIM4->ARR = axis->Speed; break;}
	case 2: case 3: {DRV8813_Dir_RA(0); TIM4->PSC = 160 - 1; TIM4->ARR = axis->Speed; break;}
	case 4: case 5: {DRV8813_Dir_RA(1); TIM4->ARR = axis->Speed; break;}
	case 6: case 7: {DRV8813_Dir_RA(0); TIM4->ARR = axis->Speed; break;}
#endif
	}
}

void ED_InitSpeedDEC (struct RegisterAxis *axis){
	if(axis->Enable) {EN_DEC_DW;}
	switch (axis->Mode){
#if defined (EQDRIVE) || (defined EQDRIVE_PRE)
	case 0: case 1: {DIR_DEC_UP; TIM3->PSC = 160 - 1; TIM3->ARR = axis->Speed; break;}
	case 2: case 3: {DIR_DEC_DW; TIM3->PSC = 160 - 1; TIM3->ARR = axis->Speed; break;}
	case 4: case 5: {DIR_DEC_UP; TIM3->ARR = axis->Speed; break;}
	case 6: case 7: {DIR_DEC_DW; TIM3->ARR = axis->Speed; break;}
#endif
#ifdef EQDRIVE_PRO
	case 0: case 1: {TIM3->PSC = 160 - 1; TIM3->ARR = axis->Speed; break;}
	case 2: case 3: {TIM3->PSC = 160 - 1; TIM3->ARR = axis->Speed; break;}
	case 4: case 5: {TIM3->ARR = axis->Speed; break;}
	case 6: case 7: {TIM3->ARR = axis->Speed; break;}
#endif
	}
}

void ED_EncoderGoto (struct RegisterAxis *axis){
	//if(*flag){
		switch (axis->Mode){
		case 1: case 5: {axis->Encoder ++; break;}
		case 3: case 7: {axis->Encoder --; break;}
		case 0: {
			axis->Encoder ++;
			if (axis->Encoder >= axis->Goto) {axis->ControlON = 0; axis->StatusGoto = 0;}
			break;
		}
		case 2: {
			axis->Encoder --;
			if (axis->Encoder <= axis->Goto) {axis->ControlON = 0; axis->StatusGoto = 0;}
			break;
		}
		case 4: {
			axis->Encoder ++;
			if (axis->Encoder >= (axis->Goto - axis->BeginBraking)) {axis->ControlON = 0; axis->StatusGoto = 0;}
			break;
		}
		case 6: {
			axis->Encoder --;
			if (axis->Encoder <= (axis->Goto + axis->BeginBraking)) {axis->ControlON = 0; axis->StatusGoto = 0;}
		}
		}
		//*flag = 0;
	//}
}

void ED_SlowStartStop(struct RegisterAxis *axis, TIM_TypeDef *timer){
	
	if (axis->ControlON){
		switch (axis->Mode){
		case 0: case 2: axis->StatusGoto = 1;
		case 1: case 3: {axis->StatusON = 1; timer->CR1 |= TIM_CR1_CEN; break;}
		case 4: case 6: axis->StatusGoto = 1;
		case 5: case 7: { //   
			axis->StatusON = 1; timer->CR1 |= TIM_CR1_CEN;
			if(((timer->PSC) > 10-1) && (axis->temp32 <= SysTickTime)){
				(timer->PSC)--;
				axis->temp32 = SysTickTime + 10;
			}
			break;
		}
		}
	}
	else {
		switch (axis->Mode){
		case 0: case 1: case 2: case 3: {
			axis->StatusON = 0;
			timer->CR1 &= ~TIM_CR1_CEN;
			timer->CNT = 0;
			break;
		}
		case 5: case 7: { //   
			if((timer->PSC) < 160-1){
			   if(axis->temp32 <= SysTickTime){
				(timer->PSC)++;
				axis->temp32 = SysTickTime + 10;
			   }
			}
			else {
				axis->StatusON = 0;
				timer->CR1 &= ~TIM_CR1_CEN;
				timer->CNT = 0;
			}
			break;
		}
		case 4: {
			if((timer->PSC) < 160-1){
			   if(axis->temp32 <= SysTickTime){
				(timer->PSC)++;
				axis->temp32 = SysTickTime + 10;
			   }
			} 
			else {
				if(!axis->StatusGoto && axis->StatusON) {axis->ControlON = 1; axis->Mode = 0;}
				else {
					axis->StatusON = 0;
					timer->CR1 &= ~TIM_CR1_CEN;
					timer->CNT = 0;
					axis->Mode = 1; axis->StatusGoto = 0;
				}
			}
			break;
		}
		case 6: {
			if((timer->PSC) < 160-1){
			   if(axis->temp32 <= SysTickTime){
				(timer->PSC)++;
				axis->temp32 = SysTickTime + 10;
			   }
			} 
			else {
				if(!axis->StatusGoto && axis->StatusON) {axis->ControlON = 1; axis->Mode = 2;}
				else {
					axis->StatusON = 0;
					timer->CR1 &= ~TIM_CR1_CEN;
					timer->CNT = 0;
					axis->Mode = 3; axis->StatusGoto = 0;
				}
			}
			break;
		}
		}
	}
}

/*	
void ED_SlowStartStop(struct RegisterAxis *axis, u16 volatile *timer){
	
	if (axis->ControlON){
		switch (axis->Mode){
		case 0: case 2: axis->StatusGoto = 1;
		case 1: case 3: {axis->StatusON = 1; break;}
		case 4: case 6: axis->StatusGoto = 1;
		case 5: case 7: { //   
			axis->StatusON = 1;
			if(((*timer) > 10-1) && (axis->temp32 <= SysTickTime)){
				(*timer)--;
				axis->temp32 = SysTickTime + 10;
			}
			break;
		}
		}
	}
	else {
		switch (axis->Mode){
		case 0: case 1: case 2: case 3: {axis->StatusON = 0; break;}
		case 5: case 7: { //   
			if((*timer) < 160-1){
			   if(axis->temp32 <= SysTickTime){
				(*timer)++;
				axis->temp32 = SysTickTime + 10;
			   }
			}
			else {axis->StatusON = 0;}
			break;
		}
		case 4: {
			if((*timer) < 160-1){
			   if(axis->temp32 <= SysTickTime){
				(*timer)++;
				axis->temp32 = SysTickTime + 10;
			   }
			} 
			else {
				if(!axis->StatusGoto && axis->StatusON) {axis->ControlON = 1; axis->Mode = 0;}
				else {axis->StatusON = 0; axis->Mode = 1; axis->StatusGoto = 0;}
			}
			break;
		}
		case 6: {
			if((*timer) < 160-1){
			   if(axis->temp32 <= SysTickTime){
				(*timer)++;
				axis->temp32 = SysTickTime + 10;
			   }
			} 
			else {
				if(!axis->StatusGoto && axis->StatusON) {axis->ControlON = 1; axis->Mode = 2;}
				else {axis->StatusON = 0; axis->Mode = 3; axis->StatusGoto = 0;}
			}
			break;
		}
		}
	}
}
*/