/*******************************************************************************
********************************************************************************
	Project	: EQDRIVE
	File	: main.c
	Version	: 0.4a
	Date	: 4.04.2013
	Author	: Ruslan Korop                           
	Company	: Home                            

MCU: STM32F100PB, board - STM32DISCOVERY.
Clock frequency	: 24,000000 MHz
********************************************************************************
*******************************************************************************/

#include    "main.h"

//Global variables
u32 SysTickTime;
//u32 PreDivTimerRA;
u8 FlagRxEnd, FlagResetTimeOut;
u8 RxBuffer[10], TxBuffer[10], RxCounter, TxCounter, TxRdIndex;

struct RegisterAxis RA, DEC;

int main(void){
//u16 temp16;
static u32 TimeOut, BlinkLedTime=0, BlinkLedPwTime=0;

	init_clock(); //   
	init_peripherals(); //   
	//SysTick_Config(2400000);
	
	ED_Init(&RA, &DEC); //         RA  DEC
	ED_InitSpeedRA(&RA);
	ED_InitSpeedDEC(&DEC);
	
    while(1){ //  
		
		// 
		switch(DRV_TestValue(&ADC1->JDR1, 0x83B, 0x680)){
		case NORM: {LED4_ON; break;}
		case WARNING: {
			if(BlinkLedPwTime <= SysTickTime) {LED4_TOGGLE; BlinkLedPwTime = SysTickTime + 400;}
			break;
		}
		case ALARM: {
			if(BlinkLedPwTime <= SysTickTime) {LED4_TOGGLE; BlinkLedPwTime = SysTickTime + 100;}
		}
		}
		
		//        199
		ED_SlowStartStop(&RA, TIMER_RA);
		ED_SlowStartStop(&DEC, TIMER_DEC);
		
		//    DEC  
		switch(DRV_SwitchVref(&DEC, &SysTickTime)){
#ifdef EQDRIVE
		case LOW: {DAC->DHR12R2 = (u16)(VREF_DEC * 0.348); break;}
		case HI : {DAC->DHR12R2 = (u16)(VREF_DEC * 1.393);}
#endif
#ifdef EQDRIVE_PRE
		case LOW: {TIM2->CCR3 = (u16)(VREF_DEC / 40); break;}
		case HI : {TIM2->CCR3 = (u16)(VREF_DEC / 10);}
#endif
		}
		
		//  
		if(RA.StatusON || DEC.StatusON){
			if(BlinkLedTime <= SysTickTime){
				LED3_TOGGLE;
				BlinkLedTime = SysTickTime + 1500;
				//if((RA.Speed == RA.Speed_1x) && (!DEC.StatusON)) BlinkLedTime = SysTickTime + 1500;
				//else  BlinkLedTime = SysTickTime + 100;
			}
		}
		else{
			if(RA.Enable) LED3_ON;
			else LED3_OF;
		}
		
		if (FlagRxEnd){ //      USART
			FlagRxEnd = 0;
			if(!(eqmod(RxBuffer, RxBuffer, &RA, &DEC))) //   
			{
				//       
				ED_InitSpeedRA(&RA);
				ED_InitSpeedDEC(&DEC);
			}
		}
		
		if (RxCounter){ //     USART1
			if(FlagResetTimeOut){
				FlagResetTimeOut = 0;
				TimeOut = SysTickTime + 10000; // Timeout = 10 c.
			}
			if(TimeOut <= SysTickTime) RxCounter = 0;
		}
	}
}

void SysTick_Handler (void){
	SysTickTime++;
}

void EXTI0_IRQHandler (void){
    //GPIOC->ODR ^= LED3;
    EXTI->PR = EXTI_PR_PR0;
}

void TIM3_IRQHandler (void){
	if(TIM3->SR & TIM_SR_UIF){
#if defined (EQDRIVE) || (defined EQDRIVE_PRE)
		STEP_DEC_UP;
#else
		//DRV8813_Step_DEC();
#endif
		ED_EncoderGoto(&DEC);
    	TIM3->SR &= ~TIM_SR_UIF;
	}
	if(TIM3->SR & TIM_SR_CC1IF){
#if defined (EQDRIVE) || (defined EQDRIVE_PRE)
		STEP_DEC_DW;
#else
		//...
#endif
		TIM3->SR &= ~TIM_SR_CC1IF;
	}
}

void TIM4_IRQHandler (void){
	if(TIM4->SR & TIM_SR_UIF){
#if defined (EQDRIVE) || (defined EQDRIVE_PRE)		
		STEP_RA_UP;
#else
		DRV8813_Step_RA(STEP_64);
#endif
		ED_EncoderGoto(&RA);
    	TIM4->SR &= ~TIM_SR_UIF;
	}
	if(TIM4->SR & TIM_SR_CC1IF){
#if defined (EQDRIVE) || (defined EQDRIVE_PRE)
		STEP_RA_DW;
#else
		//...
#endif
		TIM4->SR &= ~TIM_SR_CC1IF;
	}
}

void USART1_IRQHandler (void){ //    USART1
u32 UsartData, UsartState;

	UsartState = USART1->SR;
	UsartData  = USART1->DR; //    SR  DR 
	if (UsartState & USART_SR_RXNE){ //    
		if (!(UsartState & 0x0007)){ //    
			FlagResetTimeOut = 1; //  
			if (!RxCounter){ //    
				if (UsartData == ':') RxCounter++;
			}
			else{
				if (UsartData == '\r'){ //    
					RxBuffer[0] = RxCounter - 1;
					RxCounter = 0;
					FlagRxEnd = 1;
				}
				else RxBuffer[RxCounter++] = UsartData;
			}
		}
	}
	if (UsartState & USART_SR_TC){ //     .
		if(TxCounter){ //       
			TxCounter--;
			USART1->DR = TxBuffer[TxRdIndex++];
			if(TxRdIndex > 10) TxRdIndex = 0;
		}
		USART1->SR &= ~USART_SR_TC; //   " ".
	}
}
